/** LCA cu RMQ
   RMQ = Range Minimum Query

   Complexitate:
		Preprocesare: O(n log n)
		O(1) / query
*/

#include <bits/stdc++.h>
using namespace std;

ifstream fin("lca2.in");
ofstream fout("lca.out");

const int NMAX = 1e5 + 5, LOG = 25;
int n, m;
int N;                   // N = 2 * n - 1 - lungimea turului Euler
int E[2 * NMAX];         // nodurile parcurse in turul Euler
int niv[2 * NMAX];       // niv[i] = nivelul in arbore al nodului E[i]
int poz[2 * NMAX];       // poz[x] = poz primei aparitii a lui x in turul Euler
int p[LOG][2 * NMAX];    // p[i][j] = pozitia in sirul E a nodului cu nivel minim in intervalul [j, j + 2^i - 1],
vector<int> G[NMAX];     //           adica pe un interval de lungime 2^i care incepe la poz j

void CitesteArbore();
inline void DFS(int x, int h);
inline void RMQ();       //
inline int LCA(int x, int y);

int main()
{
    CitesteArbore();
    DFS(1, 0);
    RMQ();

	int x, y;
    while (m--)
	{
        fin >> x >> y;
		fout << LCA(x, y) << '\n';
    }
}

inline int LCA(int x, int y) /// O(1)
{
	int px = poz[x]; /// pozitiile in tutul Euler
	int py = poz[y];
	if (px > py)
		swap(px, py);

	int i = log2(py - px + 1); /// 2^i = cea mai mare putere a lui 2
                               /// mai mica sau egala cu lung intrvalului
	if (niv[p[i][px]] < niv[p[i][py - (1 << i) + 1]])
		return E[p[i][px]];

	return E[p[i][py - (1 << i) + 1]];
}

inline void DFS(int x, int nv)
{
    E[++N] = x;
    niv[N] = nv;
    poz[x] = N;

    for(int y : G[x])
    {
        DFS(y, nv + 1);
        E[++N] = x;
        niv[N] = nv;
    }
}

inline void RMQ() /// p[i][j] = poz minimului in sirul niv[]
{                 /// incepan cu poz j si de lungime 2^i
	for (int i = 1; i <= N; ++i)
		p[0][i] = i;

	int p1, p2;
    for (int i = 1; (1 << i) <= N; ++i)
        for (int j = 1; j <= N; ++j)
        {
			p1 = p[i - 1][j];
			p2 = p[i - 1][j + (1 << (i - 1))];
            if (niv[p1] < niv[p2])
                p[i][j] = p1;
            else
                p[i][j] = p2;
		}
}

void CitesteArbore()
{
	fin >> n >> m;
    for(int i = 2, tata; i <= n; ++i)
    {
        fin >> tata;
        G[tata].push_back(i);
    }
}
